#ifndef ER_H
#define ER_H

/*
  This file implements Exception Handeling in C (and C++ if you want...).

  The software may be used and modified as you wish, but this comment
  should remain a permanent part of the file.

  Yonatan Lehman
  18 Oct. 2000
*/

/*
  ER_status_t  is a typedef that defines an error status.
*/

typedef unsigned long ER_status_t;


/*
  This defines an OK status
*/
#define ER_OK 0

/*
  Change the following macro to suit the logging utility you want to use.
  The prototype is assumed to be printf compatible. But you can easily
  change the calls to ER_LOG to use whatever logging facility you use.
*/
#include "stdio.h"  //for printf prototype
#define ER_LOG   printf

/*
  put ER_ENTER at the start of each routine, after the local
  variable declarations
*/

#define ER_ENTER() {long __er_line;


/*
  Put an EXIT : label followed by cleanup code followed by the ER_EXIT
  at the end of each routine
*/

#define ER_EXIT(ST_P) {if (*(ST_P) != ER_OK)  {\
    __er_call_log(*(ST_P), __er_line);}}}

/*
  Use this to assign a status to the status parameter.
*/

#define ER_RAISE(ST_P,ST) {\
   __er_line = __LINE__;\
   ER_LOG("%s(%d) ---RAISE--- %x\n",ER_FILE(), __er_line, ST);\
   (*(ST_P)) = (ST);  goto EXIT;}

/*
  Use ER_CHK after each routine call - putting it at teh end of the line
  makes the code more readable
  foo(....st_p);ER_CHK(st_p);
 */
#define ER_CHK(ST_P) {if ((*(ST_P)) != ER_OK) {__er_line = __LINE__; goto EXIT;}}

#if 0
/*
 * you can use this version of ER_CHK if you don't have memory problems.
 * It eliminates the need to use ER_ENTER ans ER_EXIT.
 */

#define ER_CHK(ST_P) {if ((*(ST_P)) != ER_OK) \
      {__er_call_log(*(ST_P), __LINE__); goto EXIT;}}

/* you can eliminate the macro ER_DEF_FILE if you pass the __FILE__
 * directly here and call ER_LOG directly.
 */

#define ER_CHK(ST_P) {if ((*(ST_P)) != ER_OK) \
      {ER_LOG("%s(%d) return status %x\n", __FILE__, __LINE__, (*(ST_P)));
       goto EXIT;}

#endif

/*
  Use this at the point of recovery to reset the status and to log
  the end of the trace
 */
#define ER_RECOVER(ST_P) {ER_LOG("%s(%d) ---RECOVER--- %x\n",\
     ER_FILE(), __LINE__, *(ST_P));}

/*
  Set ER_DEF_FILE at the start of each file, passing it the name
  of the file (or __FILE__ if you like). Use ER_FILE instead of
  __FILE__ to save memory space
*/
#define ER_FILE() __filename
#define ER_DEF_FILE(FILESTR) static char __filename[] = FILESTR;\
    static void __er_call_log(ER_status_t status, long line) {\
       ER_LOG("%s(%d) return status %x\n", ER_FILE(), line, status);}
typedef char __er_dumy; //prevents error when ";" is put after ER_DEF_FILE


#endif
