#include "er.h"
#include "stdlib.h"

ER_DEF_FILE("er_example.c");

#define error1   1
#define error2   2

void c(int  val, ER_status_t *st_p)
{
  ER_ENTER();
  printf("val is %d\n", val);
  if (val > 0)
     ER_RAISE(st_p, error1);
  EXIT :;
  ER_EXIT(st_p);
}

void b1(int val, ER_status_t *st_p)
{
    ER_ENTER();
    
    c(val, st_p); ER_CHK(st_p);

    EXIT :;
    {
        //here is one way to check a status AFTER the EXIT
        //DON'T CALL ER_CHK since you will get an infinate loop
        ER_status_t    local_st = ER_OK;
        c(val, &local_st); ER_RECOVER(&local_st);
        //or if ypu don't want to recover write explicitly to the log
        
    }
    ER_EXIT(st_p);
}

void b2(int val, ER_status_t *st_p)
{
    int    *buffer = 0;
    ER_ENTER();
    c(10, st_p); ER_CHK(st_p);
    buffer = malloc(val);
    if (buffer == NULL)
        ER_RAISE(st_p, error2);
    c(val, st_p); ER_CHK(st_p);
    free(buffer); buffer = NULL;
    c(20, st_p); ER_CHK(st_p);
    EXIT :;
    ER_EXIT(st_p);
    if (buffer != NULL)
        free(buffer);
}

void a1(int  val, ER_status_t *st_p)
{
    ER_ENTER();

    b1(val, st_p); ER_CHK(st_p);
    
    EXIT :;
    ER_EXIT(st_p);
}

void a2(int  a, ER_status_t *st_p)
{
    ER_ENTER();

    b2(a, st_p); ER_CHK(st_p);
    
    EXIT :;
    ER_EXIT(st_p);
}

int main(int argc, char *argv[])
{
    ER_status_t    st = ER_OK; /* This is where we define and initialize the
                                  status - all pointers passed to all routines
                                  actually point to this...
                                */
    ER_status_t   *st_p = &st; /* this allows us always to use st_p as an argument
                                  which is often convveniant */
        ER_ENTER();

    //while (1)
    {
         int   j;
        for (j=0; j<5; j++)
        {
            printf("j=%d\n",j);
            if ((j % 2) == 0)
                a1(j, st_p);
            else
                a2(-j,st_p); 
            if (*st_p == error1)
            {
                printf("error1 occured - recovering\n");
                ER_RECOVER(st_p);
            }
            else if (*st_p == error2)
            {
                printf("error2 occured - recovering\n");
                ER_RECOVER(st_p);
            }
            else if (*st_p != ER_OK)
            {
                printf("unrecoverable error %x - aborting\n", *st_p);
                ER_CHK(st_p);
            }
        }
    }
    EXIT :;
    ER_EXIT(st_p);
    if (*st_p != ER_OK)
        return 1;
    else
        return 0;
}

