// Copyright (c) 1996-2000 by Vladimir Batov. Permission to  use,
// copy, modify,  distribute  and  sell  this  software  and  its
// documentation for any purpose is hereby granted  without  fee,
// provided that the above copyright notice appear in all  copies
// and that both that copyright notice and this permission notice
// appear in supporting documentation. I make no  representations
// about the suitability of this software for any purpose. It  is
// provided "as is" without express or implied warranty.

#ifndef VB_SIGNAL_PARCEL_H
#define VB_SIGNAL_PARCEL_H

namespace SignalNamespace {

struct Callee {};

typedef void (Callee::*MemberCallback) (const Event&, void*);
typedef void (*LonelyCallback) (const Event&, void*, void*);

class Parcel // Storage for callback-related data.
{
   public:

   Parcel(Event* event, Callee* receiver, MemberCallback cb)
   :
      _event(event),
      _receiver(receiver), 
      _member_callback(cb),
      _lonely_callback(0)
   {
   }

   Parcel(Event* event, Callee* receiver, LonelyCallback cb)
   :
      _event(event),
      _receiver(receiver), 
      _member_callback(0),
      _lonely_callback(cb)
   {
   }

   Parcel(const Parcel& parcel)
   :
      _event(parcel._event),
      _receiver(parcel._receiver), 
      _member_callback(parcel._member_callback),
      _lonely_callback(parcel._lonely_callback)
   {
      // Transfer ownership of the allocated event.
      ((Parcel*) &parcel)->_event = 0;
   }

   ~Parcel()
   {
      delete _event;
   }
   
   bool operator==(const Parcel& parcel) const
   {
      return (*_event          == *parcel._event)           &&
             (_receiver        ==  parcel._receiver)        &&
             (_member_callback ==  parcel._member_callback) &&
             (_lonely_callback ==  parcel._lonely_callback);
   }

   void
   call(void* sender, const Event& event) const
   {
      if (*_event == event)
      {
         _lonely_callback 
            ? _lonely_callback(event, sender, _receiver)
            : (_receiver->*_member_callback)(event, sender);
      }
   }

   private:

   Event*                   _event;
   Callee*               _receiver;
   MemberCallback _member_callback;            
   LonelyCallback _lonely_callback;            

   Parcel& operator=(const Parcel&); // Not implemented.
};
   
} // End of SignalNamespace

#endif
