// Copyright (c) 1996-2000 by Vladimir Batov. Permission to  use,
// copy, modify,  distribute  and  sell  this  software  and  its
// documentation for any purpose is hereby granted  without  fee,
// provided that the above copyright notice appear in all  copies
// and that both that copyright notice and this permission notice
// appear in supporting documentation. I make no  representations
// about the suitability of this software for any purpose. It  is
// provided "as is" without express or implied warranty.

#ifndef VB_SIGNALS_H
#define VB_SIGNALS_H

#include <stl/list.h>
#include <stl/map.h>
#include "event.h"
#include "parcel.h"

// connect() and disconnect() signatures should cover
// all 'const' variations:
//    void (Receiver::*)(const Event&,       Sender*),
//    void (Receiver::*)(const Event&, const Sender*),
//    void (*)(const Event&,       Sender*,       Receiver*),
//    void (*)(const Event&, const Sender*,       Receiver*),
//    void (*)(const Event&,       Sender*, const Receiver*),
//    void (*)(const Event&, const Sender*, const Receiver*).

#define FAMILY(function, ret_value)                            \
                                                               \
template<class Sender, class Receiver, class UserEvent>        \
static ret_value function(Sender& sender, Receiver& receiver,  \
   void (Receiver::* cb)(const UserEvent&, Sender&))           \
{                                                              \
   Parcel parcel(new UserEvent,                                \
                 (Callee*) &receiver, (MemberCallback) cb);    \
   _signals()._ ## function(&sender, parcel);                  \
}                                                              \
                                                               \
template<class Sender, class Receiver, class UserEvent>        \
static ret_value function(Sender& sender, Receiver& receiver,  \
   void (Receiver::* cb)(const UserEvent&, const Sender&))     \
{                                                              \
   Parcel parcel(new UserEvent,                                \
                 (Callee*) &receiver, (MemberCallback) cb);    \
   _signals()._ ## function(&sender, parcel);                  \
}                                                              \
                                                               \
template<class Sender, class Receiver, class UserEvent>        \
static ret_value function(Sender& sender, Receiver& receiver,  \
   void (* cb)(const Event&, Sender*, Receiver*))              \
{                                                              \
   Parcel parcel(new UserEvent,                                \
                 (Callee*) &receiver, (LonelyCallback) cb);    \
   _signals()._ ## function(&sender, parcel);                  \
}

namespace SignalNamespace {

class Signal
{
   public: ~Signal () {} 
   private: Signal () {} 

   public:
   
   typedef SignalNamespace::Event Event;
   
   template<class Sender, class UserEvent> 
   static
   void
   emit(Sender& sender, const UserEvent& event)
   {
      _signals()._emit((void*) &sender, event);
   }

   FAMILY(connect,    void)
   FAMILY(disconnect, bool)

   private:
   
   typedef list<Parcel>          List;
   typedef map<const void*, List> Map;

   Map _map;
   
   static Signal& _signals ();
   
   void       _emit (void*, const Event&);
   void    _connect (const void*, const Parcel&);
   bool _disconnect (const void*, const Parcel&);
};

} // End of SignalNamespace

typedef SignalNamespace::Signal Signal;

#endif // VB_SIGNALS_H
