// Copyright (c) 1996-2000 by Vladimir Batov. Permission to  use,
// copy, modify,  distribute  and  sell  this  software  and  its
// documentation for any purpose is hereby granted  without  fee,
// provided that the above copyright notice appear in all  copies
// and that both that copyright notice and this permission notice
// appear in supporting documentation. I make no  representations
// about the suitability of this software for any purpose. It  is
// provided "as is" without express or implied warranty.

#include "include/signals.h"

Signal&
Signal::_signals()
{
   static Signal the_instance;

   return the_instance;
}

void
Signal::_emit(void* sender, const Signal::Event& event)
{
   Map::iterator mi = _map.find(sender);

   // There are no callbacks registered for the 'sender'.
   if (mi == _map.end()) return;

   // List of callbacks registered for the 'sender'.
   List& list = mi->second;
   
   for (List::iterator li = list.begin(); li != list.end(); ++li)
   {
      li->call(sender, event);
   }
}

void
Signal::_connect(const void* sender, const Parcel& parcel)
{
   _map[sender].push_back(parcel);
}

bool
Signal::_disconnect(const void* sender, const Parcel& parcel)
{
   Map::iterator mi = _map.find(sender);

   // There are no callbacks registered for the 'sender'.
   if (mi == _map.end()) return false;

   // List of callbacks registered for the 'sender'.
   List& list = mi->second;
   
   for (List::iterator li = list.begin(); li != list.end(); ++li)
   {
      if (parcel == *li)
      {
         // Delete corresponding entries in the callback list and
         // in the map if there are no callbacks left for the
         // 'sender'.
         
         list.erase(li);
         if (list.empty()) _map.erase(mi);
         
         return true;
      }
   }
   return false;
}
