#include <iostream>
#include <fstream>
#include "point.h"
#include <time.h> // Should really be <ctime> but not all compilers put
                  // the resulting functions in namespace std...

class Timer
{
public:
  Timer( const char* name, std::ostream& os )
    : start_( clock() )
    , name_(name)
    , os_(os)
  { }
  ~Timer() { os_ << name_ << ": " << (clock() - start_) << "\n"; }
private:
  clock_t start_;
  const char* name_;
  std::ostream& os_;
};

const int iterations = 40000000;

void Test( std::ostream& os )
{
  {
    Point1 p;
    double d = 0.0;

    Timer t("1", os);
    for( int i = 0; i < iterations; ++i )
    {
      d += p.coord_[1];
    }
  }

  {
    Point2 p;
    double d = 0.0;

    Timer t("2", os);
    for( int i = 0; i < iterations; ++i )
    {
      d += p[1];
    }
  }

  {
    Point3 p;
    double d = 0.0;

    Timer t("3", os);
    for( int i = 0; i < iterations; ++i )
    {
      d += p[1];
    }
  }
}

int main()
{
  std::ofstream junk("junk");
  Test( junk ); // throw away the first results, to avoid
                // misleading inflation due to cache loading

  Test( std::cout );
  std::cout << "\n";

  Test( std::cout );
}


